import sqlite3
import json
import os
from pathlib import Path
db_file = str(Path.home().joinpath(f".indy_client/vsw/sqlite.db"))


def init():
    if not os.path.exists(str(Path(os.path.expanduser('~'))) + "/.indy_client/vsw/"):
        os.makedirs(str(Path(os.path.expanduser('~'))) + "/.indy_client/vsw/")
    conn = sqlite3.connect(db_file)
    c = conn.cursor()
    sql = 'CREATE TABLE IF NOT EXISTS credentials (issuer_did TEXT NOT NULL, software_name TEXT, software_did TEXT, status TEXT, content TEXT,  sqltime TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL);'
    c.execute(sql)
    conn.commit()
    c.close()
    conn.close()


def save_credential(issuer_did, software_name, software_did, status, jsonObject):
    conn = sqlite3.connect(db_file)
    c = conn.cursor()
    sql = f"INSERT INTO credentials (issuer_did, software_name, software_did, status, content) VALUES(?,?,?,?,?)"
    c.execute(sql, (issuer_did, software_name, software_did, status, json.dumps(jsonObject)))
    conn.commit()
    c.close()
    conn.close()


def get_credential_by_issuer_did(issuer_did):
    conn = sqlite3.connect(db_file)
    c = conn.cursor()
    sql = "select content from credentials where issuer_did = ?"
    c.execute(sql, (issuer_did,))
    results = c.fetchall()
    c.close()
    conn.close()
    return results


def get_credential_by_issuer_did_and_name(issuer_did, software_name):
    conn = sqlite3.connect(db_file)
    c = conn.cursor()
    sql = "select content from credentials where issuer_did= ? and software_name= ? "
    c.execute(sql, (issuer_did, software_name))
    results = c.fetchall()
    c.close()
    conn.close()
    return results


def remove_credential(issuer_did):
    conn = sqlite3.connect(db_file)
    c = conn.cursor()
    c.execute(f"delete from credentials where issuer_did = ?", (issuer_did))
    conn.commit()
    c.close()
    conn.close()



import argparse
from typing import List

import requests
from vsw.commands import exit
import vsw.utils
from vsw.log import Log

vsw_config = vsw.utils.get_vsw_agent()
vsw_url_host = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}'
client_header = {"x-api-key": vsw_config.get("seed")}
logger = Log(__name__).logger
timeout = 60


def main(args: List[str]) -> bool:
    try:
        if exit.check_vsw_is_running():
            args = parse_args(args)
            revocation_registry_id = args.rev_reg_id
            credential_revocation_id = args.cred_rev_id
            if not revocation_registry_id or not credential_revocation_id:
                print('vsw: error: the credential registry id and credential revocation id are both required.')
                print_help()
                return

            revoke(revocation_registry_id, credential_revocation_id, args.publish)
    except requests.exceptions.RequestException:
        logger.error(vsw.utils.Constant.NOT_RUNNING_MSG)
    except KeyboardInterrupt:
        print(" ==> Exit revoke!")


def print_help():
    print('Usage:')
    print('vsw revoke [options]')
    print('-reg, --rev_reg_id       Revocation registry identifier')
    print('-rev, --cred_rev_id      The Credential Revocation identifier')


def parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument("-rev", "--rev-reg-id", required=False, help="Revocation registry identifier")
    parser.add_argument("-cred", "--cred_rev_id", required=False, help="Credential revocation identifier")
    parser.add_argument("-p", "--publish", required=False, default=True,  help="If publish revocation immediately")

    return parser.parse_args(args)


def revoke(revocation_registry_id, credential_revocation_id, is_publish):
    publish = "false"
    if is_publish:
        publish = "true"
    url = f'{vsw_url_host}/issue-credential/revoke?rev_reg_id={revocation_registry_id}&cred_rev_id={credential_revocation_id}&publish={publish}'
    logger.info(f'The request revoke url: {url}')
    revocation_res = requests.post(url=url, headers=client_header)
    logger.info(revocation_res.__dict__)
    status_code = revocation_res.status_code
    if status_code == 200:
        print("Revoke successfully!")
    else:
        reason = revocation_res.reason
        print(reason)




import argparse
import json
import socket
import time
from typing import List

import requests

from vsw.utils import Constant
import vsw.utils
from vsw.log import Log
from multiprocessing.connection import Listener

logger = Log(__name__).logger
timeout = Constant.TIMEOUT
vsw_config = vsw.utils.get_vsw_agent()
vsw_repo_config = vsw.utils.get_repo_host()
client_header = {"x-api-key": vsw_config.get("seed")}
software_certificate = vsw_config.get("schema_name")
test_certificate = vsw_config.get("test_schema_name")


def main(args: List[str]) -> bool:
    args = parse_args(args)
    print(args)
    try:
        if args.connection:
            connection_repo()
        if args.credential_definition:
            do_credential_definition(args.schema)
    except requests.exceptions.RequestException:
        logger.error(vsw.utils.Constant.NOT_RUNNING_MSG)
    except KeyboardInterrupt:
        print(" ==> Exit init")


def get_schema_id_by_name(vsw_config, schema_name):
    if schema_name == vsw_config.get("test_schema_name"):
        return vsw_config.get("test_schema_id")
    else:
        return vsw_config.get("schema_id")


def check_credential_definition(schema_id):
    cd_check_url = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}/credential-definitions/created?schema_id={schema_id}'
    resp = requests.get(url=cd_check_url, headers=client_header)
    check_res = json.loads(resp.text)
    if len(check_res["credential_definition_ids"]) > 0:
        return check_res["credential_definition_ids"][0]
    else:
        return None


def do_credential_definition(schema_name):
    if schema_name != software_certificate and schema_name != test_certificate:
        print(f"vsw: error: the schema name must be either {software_certificate} or {test_certificate}")
        return;
    schema_id = get_schema_id_by_name(vsw_config, schema_name)
    # check if created
    check_result = check_credential_definition(schema_id)
    if check_result:
        print(f'vsw: error: creddef already existed, {check_result}')
        return;
    credential_definition_url = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}/credential-definitions'
    res = requests.post(url=credential_definition_url, json={
        "revocation_registry_size": 100,
        "support_revocation": True,
        "schema_id": schema_id,
        "tag": "default"
    }, headers=client_header)
    logger.info(res.__dict__)
    credential_definition_res = json.loads(res.text)
    logger.info(credential_definition_res)
    print(f'Created credential definition id: {credential_definition_res["credential_definition_id"]}')


def parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument("-cd", "--credential-definition", action='store_true')
    parser.add_argument('-c', '--connection', action='store_true')
    parser.add_argument('-s', '--schema', default=software_certificate, help="The schema name")

    return parser.parse_args(args)


def connection_repo():
    address = ('localhost', Constant.PORT_NUMBER)
    listener = Listener(address)
    listener._listener._socket.settimeout(Constant.TIMEOUT)
    remove_history_connection(vsw_config)
    vsw_repo_url = f'{vsw_repo_config.get("host")}/controller/invitation'
    logger.info(f'Create invitation to: {vsw_repo_url}')
    response = requests.get(url=vsw_repo_url)
    logger.info(response.__dict__)
    res = json.loads(response.text)
    logger.info(res.__dict__)
    local_url = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/connections/receive-invitation?alias={vsw_config.get("label")}'
    logger.info(f'Receive invitation {local_url}')
    invitation = res["invitation"]
    body = {
        "label": invitation["label"],
        "serviceEndpoint": invitation["serviceEndpoint"],
        "recipientKeys": invitation["recipientKeys"],
        "@id": invitation["@id"]
    }
    ss = requests.post(url=local_url, json=body, headers=client_header)
    logger.info(ss.__dict__)
    invitation_response = json.loads(ss.text)

    connection_id = invitation_response["connection_id"]
    while True:
        try:
            conn = listener.accept()
            msg = conn.recv()
            state = msg["state"]
            conn.close()
            logger.info(f'waiting state change, current state is: {state}')
            if state == 'active':
                print("Created connection successfully!")
                listener.close()
                break
            else:
                time.sleep(0.5)
        except socket.timeout as e:
            remove_connection(connection_id, vsw_config)
            logger.error(e)
            print("vsw: error: request timeout, there might be some issue during initializing connection.")
            listener.close()
            break;


def remove_history_connection(vsw_config):
    schema_url = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}/connections'
    response = requests.get(url=schema_url, headers=client_header)
    results = json.loads(response.text)["results"]
    for result in results:
        schema_url = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}/connections/{result["connection_id"]}/remove'
        requests.post(url=schema_url, headers=client_header)
        logger.info(f"Removed history connection id: {result['connection_id']}")


def remove_connection(connection_id, vsw_config):
    schema_url = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}/connections/{connection_id}/remove'
    requests.post(url=schema_url, headers=client_header)



import argparse
import calendar
import json
import socket
import time
from datetime import datetime, timezone
from multiprocessing.connection import Listener
from typing import List
from urllib.parse import urljoin

from rich.console import Console

console = Console()
import requests
import validators
from aries_cloudagent_vsw.messaging.util import str_to_epoch
from version_parser import Version
from vsw.utils import Constant
import vsw.utils
from vsw.log import Log
from vsw.commands import exit

vsw_config = vsw.utils.get_vsw_agent()
vsw_url_host = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}'
client_header = {"x-api-key": vsw_config.get("seed")}
logger = Log(__name__).logger
timeout = Constant.TIMEOUT
TITLE = "Proof of Test Certificate"


def main(args: List[str]) -> bool:
    try:
        if exit.check_vsw_is_running():
            parser = argparse.ArgumentParser(prog="vsw verify")
            parser.add_argument("-p", "--proof-request", required=True, help="The software credential json file path")
            parser.add_argument("-d", "--revoke-date", required=False, help="The revoke date, %Y-%m-%d")
            parsed_args = parser.parse_args(args)
            execute(parsed_args.proof_request, parsed_args.revoke_date)
    except requests.exceptions.RequestException:
        logger.error(vsw.utils.Constant.NOT_RUNNING_MSG)
    except KeyboardInterrupt:
        print(" ==> Exit verify!")


def execute(proof_request, revoke_date):
    with open(proof_request) as json_file:
        data = json.load(json_file)
        requested_attributes = data["requested_attributes"]
        software_request_attribute = get_software_requested_attribute(requested_attributes)
        test_request_attribute = get_test_request_attribute(requested_attributes)
        software_request_attribute = arrange_request_attribute(software_request_attribute)
        test_request_attribute = arrange_request_attribute(test_request_attribute)

        if software_request_attribute and software_request_attribute.get("restrictions"):
            for restriction in software_request_attribute.get("restrictions"):
                if "attr::softwareversion::value" in restriction:
                    software_version = restriction["attr::softwareversion::value"]
                    if check_version(software_version) is False:
                        return;
                if "attr::softwareurl::value" in restriction:
                    software_url = restriction["attr::softwareurl::value"]
                    if not validators.url(software_url):
                        print('vsw: error: the software package url is wrong, please check')
                        return

        requested_predicates = {}
        if "requested_predicates" in data:
            requested_predicates = data["requested_predicates"]
        connection = get_client_connection()
        logger.info(f'issuer connection_id: {connection["connection_id"]}')
        address = ('localhost', Constant.PORT_NUMBER)
        listener = Listener(address)
        listener._listener._socket.settimeout(Constant.TIMEOUT)
        proof_response = send_request(connection["connection_id"], software_request_attribute, test_request_attribute,
                                      requested_predicates, revoke_date)
        presentation_exchange_id = proof_response["presentation_exchange_id"]
        logger.info(f'presentation_exchange_id: {presentation_exchange_id}')

        while True:
            try:
                conn = listener.accept()
                msg = conn.recv()
                state = msg["state"]
                conn.close()
                logger.info(f"waiting state update, current state is: {state}")
                if state == "verified":
                    console.print("======presentation_request========")
                    console.print(json.dumps(msg["presentation_request"], indent=4, sort_keys=True))
                    console.print("======requested_proof========")
                    console.print(json.dumps(msg["presentation"]["requested_proof"], indent=4, sort_keys=True))
                    if msg["verified"] == "false":
                        remove_proof_request(presentation_exchange_id)
                        print("Verified error, Verified result from indy is False!")
                        listener.close()
                        break;
                    pres_req = msg["presentation_request"]
                    is_proof_of_software_certificate = (
                            pres_req["name"] == TITLE
                    )
                    if is_proof_of_software_certificate:
                        print('Congratulation! verified successfully!')
                    else:
                        remove_proof_request(presentation_exchange_id)
                        print("Verified error, the name in presentation request is wrong")
                    listener.close()
                    break;
                else:
                    time.sleep(0.5)
            except socket.timeout as e:
                remove_proof_request(presentation_exchange_id)
                print("Request timeout, Verified error!")
                logger.error(e)
                listener.close()
                break;


def arrange_request_attribute(request_attribute):
    if not request_attribute:
        return None
    restrictions = request_attribute.get("restrictions")
    arranged_restrictions = []
    for restriction in restrictions:
        restriction = remove_empty_from_dict(restriction)
        restriction = replace_attr_fields(restriction)
        arranged_restrictions.append(restriction)
    request_attribute["restrictions"] = arranged_restrictions
    return request_attribute


def remove_empty_from_dict(d):
    if type(d) is dict:
        return dict((k, remove_empty_from_dict(v)) for k, v in d.items() if v and remove_empty_from_dict(v))
    elif type(d) is list:
        return [remove_empty_from_dict(v) for v in d if v and remove_empty_from_dict(v)]
    else:
        return d


def replace_attr_fields(restriction : dict):
    new_restriction = {}
    for key, value in restriction.items():
        if key not in ["schema_id", "schema_issuer_did", "schema_name", "schema_version",
                       "cred_def_id", "issuer_did"]:
            new_restriction[f'attr::{key.lower()}::value'] = value
        else:
            new_restriction[key] = value
    return new_restriction


def get_software_requested_attribute(requested_attributes):
    for req_attr in requested_attributes:
        restrictions = req_attr["restrictions"]
        for restriction in restrictions:
            if restriction["schema_id"] == vsw_config.get("schema_id"):
                return req_attr
    return None


def get_test_request_attribute(requested_attributes):
    for req_attr in requested_attributes:
        restrictions = req_attr["restrictions"]
        for restriction in restrictions:
            if restriction["schema_id"] == vsw_config.get("test_schema_id"):
                return req_attr
    return None


def check_version(software_version):
    try:
        Version(software_version)
    except ValueError:
        print("vsw: error: the software version format is incorrect. the correct format should be 'MAJOR.MINOR.PATCH'")
        return False
    return True


def remove_proof_request(presentation_exchange_id):
    vsw_url = urljoin(vsw_url_host, f"/present-proof/records/{presentation_exchange_id}/remove")
    requests.post(url=vsw_url, headers=client_header)


def retrieve_result(presentation_exchange_id):
    time.sleep(1)  # wait communicate complete automatically between agents
    presentation_proof_result = get_vsw_proof(presentation_exchange_id)
    return presentation_proof_result


def get_vsw_proof(pres_ex_id):
    vsw_url = urljoin(vsw_url_host, f"/present-proof/records/{pres_ex_id}")
    res = requests.get(url=vsw_url, headers=client_header)
    return json.loads(res.text)


def send_request(client_conn_id, software_request_attribute, test_request_attribute, requested_predicates, revoke_date):
    vsw_url = f'{vsw_url_host}/present-proof/send-request'
    NOW_8601 = datetime.utcnow().replace(tzinfo=timezone.utc).isoformat(" ", "seconds")
    NOW_EPOCH = str_to_epoch(NOW_8601)
    time_from = 0
    time_to = NOW_EPOCH

    if revoke_date:
        time_to = calendar.timegm(datetime.strptime(revoke_date, '%Y-%m-%d').timetuple())

    request_attributes = {}
    if software_request_attribute:
        request_attributes["0_software_certificate_uuid"] = {
            "names": list(map(str.lower,software_request_attribute["names"])),
            "non_revoked": {"from": time_from, "to": time_to},
            "restrictions": software_request_attribute["restrictions"]
        }
    if test_request_attribute:
        request_attributes["1_test_certificate_uuid"] = {
            "names": list(map(str.lower,test_request_attribute["names"])),
            "non_revoked": {"from": time_from, "to": time_to},
            "restrictions": test_request_attribute["restrictions"]
        }
    if requested_predicates:
        for p in requested_predicates.values():
            for key, value in p.items():
                if key == "restrictions":
                    arranged_restrictions = []
                    for rp_restriction in value:
                        arranged_restrictions.append(replace_attr_fields(rp_restriction))
                    p["restrictions"] = arranged_restrictions
                    break;
            p["non_revoked"] = {"from": time_from, "to": time_to}

    indy_proof_request = {
        "name": TITLE,
        "version": "1.0",
        "requested_attributes": request_attributes,
        "requested_predicates": requested_predicates
    }
    proof_request_web_request = {
        "connection_id": client_conn_id,
        "proof_request": indy_proof_request
    }
    logger.info(json.dumps(proof_request_web_request))
    res = requests.post(url=vsw_url, json=proof_request_web_request, headers=client_header)
    logger.info(res.__dict__)
    return json.loads(res.text)


def get_client_connection():
    connection_response = requests.get(url=f'{vsw_url_host}/connections?state=active', headers=client_header)
    res = json.loads(connection_response.text)
    logger.info(res.__dict__)
    connections = res["results"]
    if len(connections) > 0:
        return connections[-1]
    else:
        raise ConnectionError("vsw: error: Not found active vsw connection! Have you executed vsw setup connection?")



import argparse
import json
from typing import List
from urllib.parse import urljoin
from rich.console import Console
import requests
from vsw.dao import vsw_dao
from vsw.log import Log
from vsw.utils import get_vsw_agent, Constant
from vsw.commands import exit
logger = Log(__name__).logger
console = Console()
vsw_config = get_vsw_agent()
client_header = {"x-api-key": vsw_config.get("seed")}


def main(argv: List[str]) -> bool:
    if exit.check_vsw_is_running():
        args = parse_args(argv)
        try:
            if args.connection:
                get_connections(vsw_config)
            elif args.wallet:
                get_wallet(vsw_config)
            elif args.schema:
                get_schema(vsw_config)
            elif args.status:
                get_status(vsw_config)
            elif args.credentials:
                get_credentials(vsw_config)
            elif args.credential_definition:
                get_credential_definition(vsw_config)
            elif args.present_proof:
                get_present_proof(vsw_config)
            else:
                console.print('Usage:')
                console.print('vsw list [options]')
                console.print('-c: list all connections')
                console.print('-w: list all DIDs in the wallet')
                console.print('-sc: list all supported schema')
                console.print('-s: show agent status')
                console.print('-p: list all presentation proof records')
                console.print('-cs: list all credentials issued by the current DID')
                console.print('-cd: list all credential definitions registered by the current DID')
        except KeyboardInterrupt:
            print(" ==> Exit list!")
        except requests.exceptions.RequestException:
            logger.error(Constant.NOT_RUNNING_MSG)
        except Exception as e:
            logger.error("Failed to execute list: " + str(e))


def parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument('-c', '--connection', action='store_true')
    parser.add_argument('-w', '--wallet', action='store_true')
    parser.add_argument('-sc', '--schema', action='store_true')
    parser.add_argument('-s', '--status', action='store_true')
    parser.add_argument('-p', '--present_proof', action='store_true')
    parser.add_argument('-cs', '--credentials', action='store_true')
    parser.add_argument('-cd', '--credential_definition', action='store_true')
    return parser.parse_args(args)


def get_credential_definition(vsw_config):
    did = get_public_did(vsw_config)
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/credential-definitions/created?issuer_did={did}'
    response = requests.get(url=local, headers=client_header)
    res = json.loads(response.text)
    console.print(json.dumps(res, indent=4, sort_keys=True))


def get_credentials(vsw_config):
    did = get_public_did(vsw_config)
    res = vsw_dao.get_credential_by_issuer_did(did)
    results = []
    for row in res:
        json_object = json.loads(row[0])
        results.append(json_object)

    console.print(json.dumps({
        "results": results
    }, indent=4, sort_keys=True))


def get_public_did(vsw_config):
    url = urljoin(f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}', "/wallet/did/public")
    response = requests.get(url=url, headers=client_header)
    res = json.loads(response.text)
    return res["result"]["did"]


def get_status(vsw_config):
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/status'
    response = requests.get(url=local, headers=client_header)
    res = json.loads(response.text)
    console.print(json.dumps(res, indent=4, sort_keys=True))


def get_schema(vsw_config):
    schema_id = vsw_config.get("schema_id")
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/schemas/{schema_id}'
    response = requests.get(url=local, headers=client_header)
    res = json.loads(response.text)
    console.print(f"======vsw software certificate schema_id: {schema_id}======")
    console.print(json.dumps(res, indent=4, sort_keys=True))

    test_schema_id = vsw_config.get("test_schema_id")
    test_local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/schemas/{test_schema_id}'
    test_response = requests.get(url=test_local, headers=client_header)
    test_res = json.loads(test_response.text)
    console.print(f"======vsw attest schema_id: {test_schema_id}======")
    console.print(json.dumps(test_res, indent=4, sort_keys=True))


def get_wallet(vsw_config):
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/wallet/did'
    response = requests.get(url=local, headers=client_header)
    res = json.loads(response.text)
    console.print(json.dumps(res, indent=4, sort_keys=True))


def get_present_proof(vsw_config):
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/present-proof/records'
    response = requests.get(url=local, headers=client_header)
    res = json.loads(response.text)
    sorted_res = res["results"]
    sorted_res.sort(key=lambda p: p["created_at"])
    if len(sorted_res) > 0:
        console.print("======presentation_request========")
        console.print(json.dumps(sorted_res[-1]["presentation_request"], indent=4, sort_keys=True))
        console.print("======requested_proof========")
        console.print(json.dumps(sorted_res[-1]["presentation"]["requested_proof"], indent=4, sort_keys=True))
    else:
        console.print(json.dumps(res))


def get_connections(vsw_config):
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/connections'
    response = requests.get(url=local, headers=client_header)
    res = json.loads(response.text)
    console.print(json.dumps(res, indent=4, sort_keys=True))


import argparse
import json
import socket
import time
from typing import List
from vsw.log import Log
from urllib.parse import urljoin
from vsw.dao import vsw_dao
import vsw.utils
import requests
import validators
from vsw.utils import Constant
from version_parser import Version
from multiprocessing.connection import Listener
from vsw.commands import attest,exit

vsw_config = vsw.utils.get_vsw_agent()
software_certificate = vsw_config.get("schema_name")
test_certificate = vsw_config.get("test_schema_name")
vsw_url_host = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}'
client_header = {"x-api-key": vsw_config.get("seed")}
logger = Log(__name__).logger
timeout = Constant.TIMEOUT
developer_did = ''
software_name = ""


def main(args: List[str]) -> bool:
    try:
        if exit.check_vsw_is_running():
            args = parse_args(args)
            with open(args.cred_file) as json_file:
                data = json.load(json_file)
                schema_name = args.schema
                logger.info(f'schema name: {schema_name}')
                if schema_name != software_certificate and schema_name != test_certificate:
                    print(f"vsw: error: the schema name must be either {software_certificate} or {test_certificate}")
                    return;
                if args.schema == vsw_config.get("test_schema_name"):
                    attest.publish(data)
                else:
                    global software_name
                    software_name = data.get("softwareName", "")
                    if "softwareName" not in data:
                        print("vsw: error: softwareName is mandatory")
                        return
                    if "softwareVersion" not in data:
                        print("vsw: error: softwareVersion is mandatory")
                        return
                    if "softwareUrl" not in data:
                        print("vsw: error: softwareUrl is mandatory")
                        return
                    if "softwareVersion" in data:
                        software_version = data["softwareVersion"]
                        if check_version(software_version) is False:
                            return;
                    if "softwareUrl" in data:
                        software_url = data["softwareUrl"]
                        if software_url and not validators.url(software_url):
                            print('vsw: error: the software package url is wrong, please check')
                            return
                    issue_credential(data)
    except requests.exceptions.RequestException:
        logger.error(vsw.utils.Constant.NOT_RUNNING_MSG)
    except ValueError as ve:
        logger.error(ve)
        print(f"vsw: error: {ve}")
    except KeyboardInterrupt:
        print(" ==> Exit publish!")


def parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument("-c", "--cred-file", required=True, help="The software credential json file path")
    parser.add_argument('-s', '--schema', default=software_certificate, help="The schema name")
    return parser.parse_args(args)


def check_version(software_version):
    try:
        Version(software_version)
    except ValueError:
        print("vsw: error: the software version format is incorrect. the correct format should be 'MAJOR.MINOR.PATCH'")
        return False
    return True


def issue_credential(data):
    address = ('localhost', Constant.PORT_NUMBER)
    listener = Listener(address)
    listener._listener._socket.settimeout(Constant.TIMEOUT)
    proposal_response = send_offer(data)
    credential_exchange_id = proposal_response["credential_exchange_id"]
    logger.info(f'credential_exchange_id: {credential_exchange_id}')
    while True:
        try:
            conn = listener.accept()
            msg = conn.recv()
            state = msg["state"]
            logger.info(f'waiting state change, current state is: {state}')
            conn.close()
            if state == 'credential_acked':
                print("Congratulation, execute publish successfully!")
                listener.close()
                break
            else:
                time.sleep(0.5)
        except socket.timeout as e:
            logger.error(e)
            print("vsw: error: request timeout, there might be some issue during publishing")
            listener.close();
            break;


def get_connection():
    vsw_connection_response = requests.get(url=f'{vsw_url_host}/connections?state=active', headers=client_header)
    res = json.loads(vsw_connection_response.text)
    connections = res["results"]
    if len(connections) > 0:
        last_connection = connections[-1]
        return last_connection
    else:
        raise ConnectionError("Not found active vsw connection! Have you executed vsw setup connection?")


def get_public_did():
    url = urljoin(vsw_url_host, "/wallet/did/public")
    response = requests.get(url=url, headers=client_header)
    res = json.loads(response.text)
    return res["result"]["did"]


def get_credentials():
    res = vsw_dao.get_credential_by_issuer_did_and_name(developer_did, software_name)
    credentials = []
    for row in res:
        json_object = json.loads(row[0])
        credentials.append(json_object)
    return credentials


def is_same_version(software_version, exist_software_version):
    if exist_software_version is None:
        return False
    v1 = Version(software_version)
    v2 = Version(exist_software_version)
    if v1.get_major_version() == v2.get_major_version() and v1.get_minor_version() == v2.get_minor_version():
        return True
    else:
        return False


def generate_software_did(software_version, download_url):
    credentials = get_credentials()
    if len(credentials) > 0:
        logger.info(f'The software name {software_name} is existed.')
        same_version = False
        for credential in credentials:
            attrs = credential["attrs"]
            same_version = is_same_version(software_version, attrs["softwareVersion"])
            if same_version:
                logger.info(f'Existed softwareDid: {attrs["softwareDid"]}')
                return attrs["softwareDid"]
        if same_version is False:
            return generate_new_did(download_url)
    else:
        return generate_new_did(download_url)


def generate_new_did(download_url):
    # Create a DID
    create_did_res = requests.post(url=urljoin(vsw_url_host, "/wallet/did/create"), headers=client_header)
    res = json.loads(create_did_res.text)
    did = res["result"]["did"]
    verkey = res["result"]["verkey"]
    # Write DID to ledger by NYM
    ledger_res = requests.post(url=urljoin(vsw_url_host, f"/ledger/register-nym?did={did}&verkey={verkey}"), headers=client_header)
    write_did_ledger_res = json.loads(ledger_res.text)
    if write_did_ledger_res["success"] is False:
        logger.err("vsw: error: write did to ledger failed!")
        raise Exception('write did to ledger failed!')
    # Set DID Endpoint
    did_endpoint_res = requests.post(url=urljoin(vsw_url_host, f"/wallet/set-did-endpoint"), json={
        "did": did,
        "endpoint_type": "Endpoint",
        "endpoint": download_url  # not support :h1 format
    }, headers=client_header)
    logger.info(f'Created new software-did: {did}')
    return did


def get_credential_definition_id():
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/credential-definitions/created?schema_id={vsw_config.get("schema_id")}'
    response = requests.get(url=local, headers=client_header)
    res = json.loads(response.text)
    if len(res["credential_definition_ids"]) == 0:
        raise ValueError('Not found credential definition id!')
    cred_def_id = res["credential_definition_ids"][-1]
    logger.info(f'cred_def_id: {cred_def_id}')
    return cred_def_id


def send_offer(data):
    global developer_did
    developer_did = get_public_did()
    connection = get_connection()
    logger.info(f'vsw client connection_id: {connection["connection_id"]}')

    digest = vsw.utils.generate_digest(data["softwareUrl"])
    source_url = data.get("sourceUrl", "")
    source_hash = vsw.utils.generate_digest(source_url)
    if "sourceHash" in data and data["sourceHash"] != source_hash:
        raise ValueError("vsw: error: incorrect sourceHash value!")

    cred_def_id = get_credential_definition_id()

    software_did = generate_software_did(data["softwareVersion"], data["softwareUrl"])
    send_offer_url = f'{vsw_url_host}/issue-credential/send-offer'
    res = requests.post(url=send_offer_url, headers=client_header, json={
        "comment": "execute vsw publish cli",
        "auto_remove": False,
        "trace": True,
        "connection_id": connection["connection_id"],
        "schema_id": data["schemaID"] or vsw_config.get("schema_id"),
        "schema_name": data["schemaName"] or vsw_config.get("schema_name"),
        "schema_version": data["schemaVersion"] or vsw_config.get("schema_version"),
        "issuer_did": developer_did,
        "cred_def_id": cred_def_id,
        "credential_preview": {
            "@type": f"did:sov:{developer_did};spec/issue-credential/1.0/credential-preview",
            "attributes": [
                {
                    "name": "developerDid",
                    "value": developer_did
                },
                {
                    "name": "softwareName",
                    "value": data["softwareName"]
                },
                {
                    "name": "softwareVersion",
                    "value": data["softwareVersion"]
                },
                {
                    "name": "softwareDid",
                    "value": software_did
                },
                {
                    "name": "softwareUrl",
                    "value": data["softwareUrl"]
                },
                {
                    "name": "softwareHash",
                    "value": digest
                },
                {
                    "name": "mediaType",
                    "value": data.get("mediaType", "")
                },
                {
                    "name": "sourceDid",
                    "value": data.get("sourceDid", "")
                },
                {
                    "name": "sourceUrl",
                    "value": source_url
                },
                {
                    "name": "sourceHash",
                    "value": source_hash
                },
                {
                    "name": "builderToolDidList",
                    "value": data.get("builderToolDidList", "")
                },
                {
                    "name": "dependencyDidList",
                    "value": data.get("dependencyDidList", "")
                },
                {
                    "name": "buildLog",
                    "value": data.get("buildLog", "")
                },
                {
                    "name": "builderDid",
                    "value": data.get("builderDid", "")
                }
            ]
        },
    })
    logger.info(res.__dict__)
    return json.loads(res.text)



import logging
import os
from multiprocessing.connection import Client
from pathlib import Path
from wsgiref.simple_server import make_server
from vsw.dao import vsw_dao
from flask import Flask, request, Response

from vsw import utils
from vsw.utils import Constant
app = Flask(__name__)

log_folder = str(Path(os.path.expanduser('~')).joinpath("vsw_logs").resolve())
if not os.path.exists(log_folder):
    os.makedirs(log_folder)
controller_log_file = str(Path(os.path.expanduser('~')).joinpath("vsw_logs/vsw-controller.log").resolve())
logging.basicConfig(filename=controller_log_file, level=logging.DEBUG)


@app.route('/')
def hello():
    return "Welcome to vsw controller"


@app.route('/webhooks/topic/connections/', methods=['POST'])
def connections():
    app.logger.info("=======/webhooks/topic/connections/ START===========")
    app.logger.info(request.json)
    app.logger.info("=======/webhooks/topic/connections/ END===========")
    address = ('localhost', Constant.PORT_NUMBER)
    conn = Client(address)
    conn.send(request.json)
    conn.close()
    return Response(status=200)


@app.route("/webhooks/topic/issue_credential/", methods=['POST'])
def issue_credential():
    app.logger.info("=======/webhooks/topic/issue_credential/ START===========")
    app.logger.info(request.json)  # Handle webhook request here
    app.logger.info("=======/webhooks/topic/issue_credential/ END===========")
    save_credential(request.json)
    address = ('localhost', Constant.PORT_NUMBER)
    conn = Client(address)
    conn.send(request.json)
    return Response(status=200)


@app.route('/webhooks/topic/present_proof/', methods=['POST'])
def present_proof():
    app.logger.info("=======/webhooks/topic/present_proof/ START===========")
    app.logger.info(request.json)  # Handle webhook request here
    app.logger.info("=======/webhooks/topic/present_proof/ END===========")
    address = ('localhost', Constant.PORT_NUMBER)
    conn = Client(address)
    conn.send(request.json)
    return Response(status=200)


@app.route("/webhooks/topic/problem_report/", methods=['POST'])
def problem_report():
    app.logger.info("=======/webhooks/topic/problem_report/ START===========")
    app.logger.info(request.json)  # Handle webhook request here
    app.logger.info("=======/webhooks/topic/problem_report/ END===========")
    address = ('localhost', Constant.PORT_NUMBER)
    conn = Client(address)
    conn.send(request.json)
    return Response(status=200)


@app.route("/webhooks/topic/revocation_registry/", methods=['POST'])
def revocation_registry():
    app.logger.info("=======/webhooks/topic/revocation_registry/ START===========")
    app.logger.info(request.json)  # Handle webhook request here
    app.logger.info("=======/webhooks/topic/revocation_registry/ END===========")
    return Response(status=200)


@app.route("/webhooks/topic/issuer_cred_rev/", methods=['POST'])
def issuer_cred_rev():
    app.logger.info("=======/webhooks/topic/issuer_cred_rev/ START===========")
    app.logger.info(request.json)  # Handle webhook request here
    app.logger.info("=======/webhooks/topic/issuer_cred_rev/ END===========")
    return Response(status=200)


@app.route("/webhooks/topic/basicmessages/", methods=['POST'])
def basicmessages():
    app.logger.info("=======/webhooks/topic/basicmessages/ START===========")
    app.logger.info(request.json)  # Handle webhook request here
    app.logger.info("=======/webhooks/topic/basicmessages/ END===========")
    return Response(status=200)


@app.route("/webhooks/topic/ping/", methods=['POST'])
def ping():
    app.logger.info(request.json)  # Handle webhook request here
    return Response(status=200)


def save_credential(msg):
    state = msg["state"]
    if state == 'credential_acked':
        credential = msg["credential"]
        revocation_id = msg.get("revocation_id", '')
        values = credential["values"]
        attrs = {}
        for key, value in values.items():
            attrs[key] = value.get("raw", "")
        content = {
            "schema_id": credential["schema_id"],
            "cred_def_id": credential["cred_def_id"],
            "rev_reg_id": credential["rev_reg_id"],
            "cred_rev_id": revocation_id,
            "attrs": attrs
        }
        issuer_did = attrs.get("developerDid", "")
        software_name = attrs.get("softwareName", "")
        software_did = attrs.get("softwareDid", "")
        vsw_dao.save_credential(issuer_did, software_name, software_did, "success", content)


if __name__ == '__main__':
    configuration = utils.get_vsw_agent()
    webhook_port = configuration.get("webhook_port")
    app.logger.info(f"Started controller, http://127.0.0.1:{webhook_port}")
    server = make_server('127.0.0.1', int(webhook_port), app)
    server.serve_forever()
    app.run()




import sys
from pathlib import Path

from aries_cloudagent_vsw.commands import run_command

from vsw import utils
from vsw.log import Log

logger = Log(__name__).logger


def start_agent(argv):
    wallet_name = argv[1]
    wallet_key = argv[2]
    seed = argv[3]
    configuration = utils.get_vsw_agent()
    config_path = Path(__file__).parent.parent.joinpath("conf/genesis.txt").resolve()
    admin_port = configuration.get("admin_port")
    transport_port = configuration.get("inbound_transport_port")
    logger.info('genesis_file: ' + str(config_path))
    try:
        args = ['--admin', configuration.get("admin_host"), admin_port,
                '--inbound-transport', configuration.get("inbound_transport_protocol"),
                configuration.get("inbound_transport_host"), transport_port,
                '--outbound-transport', configuration.get('outbound_transport_protocol'),
                '--endpoint', configuration.get("endpoint"),
                '--label', configuration.get("label"),
                '--seed', seed,
                '--webhook-url', configuration.get("webhook_url"),
                '--tails-server-base-url', utils.get_tails_server().get("host"),
                '--genesis-file', str(config_path),
                '--accept-taa', '1',
                '--wallet-type', 'indy',
                '--wallet-name', wallet_name,
                '--wallet-key', wallet_key,
                '--log-config', logger.aries_config_path,
                '--log-file', logger.aries_log_file,
                '--auto-accept-invites',
                '--preserve-exchange-records',
                '--auto-accept-requests',
                '--auto-ping-connection',
                '--auto-respond-messages',
                '--auto-respond-credential-proposal',
                '--auto-respond-credential-offer',
                '--auto-respond-credential-request',
                '--auto-store-credential',
                '--auto-respond-presentation-request',
                '--auto-verify-presentation',
                '--auto-respond-presentation-proposal',
                '--admin-api-key', seed]
        run_command('start', args)
    except BaseException as error:
        logger.error("started vsw failed.")
        logger.error('An exception occurred: {}'.format(error))


if __name__ == '__main__':
    start_agent(sys.argv)